#include <Arduino.h>
#include <Wire.h>
#include <LiquidCrystal_I2C.h>

#define CLIPPING 1500

// Define the pins for the potentiometers
const int potPin1 = 34; // First potentiometer connected to GPIO 34
const int potPin2 = 35; // Second potentiometer connected to GPIO 35

// Define the pin for the PWM output
const int pwmPin = 23; // PWM signal on GPIO 23

// Define PWM properties
const int pwmFreq = 5000;   // PWM frequency
const int pwmChannel = 0;   // PWM channel (0-15)
const int pwmResolution = 8; // PWM resolution (1-16 bits)

// Read potentiometer values
int potValue1 = 0;
int lastPotValue1 = 0;
int pwmValue1 = 0;

int potValue2 = 0;
int lastPotValue2 = 0;
int pwmValue2 = 0;

// Define a threshold for detecting significant changes
const int changeThreshold = 30;

// Initialize the LCD
LiquidCrystal_I2C lcd(0x27, 16, 2); // Change 0x27 to the address of your LCD

// Define the pins for the 4x4 button matrix
const int colPins[4] = {16, 4, 2, 15}; // Column pins
const int rowPins[4] = {19, 18, 5, 17}; // Row pins

// Array to store the state of the buttons
bool buttonStates[4][4] = {0};

// Define the frequencies for the notes (in Hz)
enum Notes {
  NOTE_C4 = 262,
  NOTE_D4 = 294,
  NOTE_E4 = 330,
  NOTE_F4 = 349,
  NOTE_G4 = 392,
  NOTE_A4 = 440,
  NOTE_B4 = 494,
  NOTE_C5 = 523,
  NOTE_D5 = 587,
  NOTE_E5 = 659,
  NOTE_F5 = 698,
  NOTE_G5 = 784,
  NOTE_A5 = 880,
  NOTE_B5 = 988,
  NOTE_C6 = 1047,
  NOTE_D6 = 1175
};

// Array to map button indices to notes
const Notes buttonNotes[16] = {
  NOTE_C4, NOTE_D4, NOTE_E4, NOTE_F4,
  NOTE_G4, NOTE_A4, NOTE_B4, NOTE_C5,
  NOTE_D5, NOTE_E5, NOTE_F5, NOTE_G5,
  NOTE_A5, NOTE_B5, NOTE_C6, NOTE_D6
};

void playNote(Notes note, int pitchModifierUp, int pitchModifierDown) {
  int period = 1000000 / note; // Calculate the period in microseconds
  unsigned long start = millis();
  while (millis() - start < 500) { // Play note for 500 milliseconds
    dacWrite(25, 255); // Set DAC output to max
    delayMicroseconds(period / 2 + pitchModifierDown);
    dacWrite(25, 0); // Set DAC output to min
    delayMicroseconds(period / 2 - pitchModifierUp);
  }
}

void setup() {
  // Configure serial communication
  Serial.begin(115200);

  // Configure the potentiometer pins as inputs
  pinMode(potPin1, INPUT);
  pinMode(potPin2, INPUT);

  // Configure PWM
  ledcSetup(pwmChannel, pwmFreq, pwmResolution);
  ledcAttachPin(pwmPin, pwmChannel);

  // Initialize the LCD with correct parameters
  lcd.init();
  lcd.begin(16, 2);
  lcd.backlight();
  lcd.clear();
  lcd.print("Bruh!");

  // Initialize button matrix pins
  for (int i = 0; i < 4; i++) {
    pinMode(colPins[i], OUTPUT);
    digitalWrite(colPins[i], HIGH); // Set columns to HIGH
    pinMode(rowPins[i], INPUT_PULLUP); // Enable internal pull-up resistors for rows
  }
}

void loop() {


  // Read the first potentiometer value (0-4095)
  potValue1 = analogRead(potPin1) / 2.75;
  //potValue1 > CLIPPING ? potValue1 = CLIPPING : potValue1 = analogRead(potPin1);

  // Read the second potentiometer value (0-4095)
  potValue2 = analogRead(potPin2) / 2.75;
  //potValue2 > CLIPPING ? potValue2 = CLIPPING : potValue2 = analogRead(potPin2);

  // Check if the change in the first potentiometer value is significant
  if (abs(potValue1 - lastPotValue1) > changeThreshold) {
    // Map the first potentiometer value to the PWM range (0-255)
    pwmValue1 = 255;//map(potValue1, 0, 4095, 0, 255);

    // Display "Pot1" on the LCD
    lcd.clear();
    lcd.print("Pot1");
  } else {
    // Set pwmValue1 to zero if the change is not significant
    pwmValue1 = 0;
    lcd.clear();
    lcd.print(potValue1 - potValue2);
  }

  // Check if the change in the second potentiometer value is significant
  if (abs(potValue2 - lastPotValue2) > changeThreshold) {
    // Map the second potentiometer value to the PWM range (0-255)
    pwmValue2 = 255; //map(potValue2, 0, 4095, 0, 255);

    // Display "Pot2" on the LCD
    lcd.clear();
    lcd.print("Pot2");
  } else {
    // Set pwmValue2 to zero if the change is not significant
    pwmValue2 = 0;
    lcd.clear();
    lcd.print(potValue1 - potValue2);
  }

  // Scan the button matrix
  for (int col = 0; col < 4; col++) {
    digitalWrite(colPins[col], LOW); // Set current column to LOW
    for (int row = 0; row < 4; row++) {
      if (digitalRead(rowPins[row]) == LOW) { // Check if button is pressed
        buttonStates[row][col] = true;
        // Handle button press
        int buttonIndex = row * 4 + col;
        Notes note = buttonNotes[buttonIndex];
        playNote(note, potValue1, potValue2); // Play the corresponding note
        lcd.clear();
        lcd.print("Button ");
        lcd.print(buttonIndex + 1); // Display button number
        ledcWrite(pwmChannel, 255);
        delay(100); // Debounce delay
        //break;
      } else {
        buttonStates[row][col] = false;
      }
    }
    digitalWrite(colPins[col], HIGH); // Set current column back to HIGH
  }

  // Determine the PWM value to be written to the output pin
  int pwmValue = max(pwmValue1, pwmValue2);

  // Set the PWM duty cycle
  ledcWrite(pwmChannel, pwmValue);

  // Update the last potentiometer values
  lastPotValue1 = potValue1;
  lastPotValue2 = potValue2;

  // Wait a bit before the next loop
  //delay(10);
}